#include "StdAfx.h"
#include "Game.h"

CGame::CGame(void)
{
	SIrrlichtCreationParameters params = SIrrlichtCreationParameters();

	params.DriverType = video::EDT_DIRECT3D9;
	params.WindowSize = irr::core::dimension2di(1024, 768);
	params.Bits = 16;
	params.Stencilbuffer = true;
	params.Fullscreen = false;
	params.Vsync = true;
	params.EventReceiver = this;

	m_Device = createDeviceEx(params);
}

void CGame::LoadScene(void) {
	video::IVideoDriver* driver = m_Device->getVideoDriver();
	scene::ISceneManager* smgr = m_Device->getSceneManager();

	// create the ground
	scene::IAnimatedMesh *planemesh=smgr->addHillPlaneMesh("plane",  
		core::dimension2df(100,100),core::dimension2d<irr::u32>(10,10),0,0	,
		core::dimension2df(0.0f, 0.0f),  
		core::dimension2df(10.0f, 10.0f) 
		);

	scene::ISceneNode *plane=smgr->addAnimatedMeshSceneNode(planemesh);

	irr::video::ITexture *plane_texture = driver->getTexture("./plane.jpg");
	plane->setMaterialTexture(0,plane_texture);
	//	node->setDebugDataVisible(scene::EDS_FULL);

	TA::AABB world(TA::Vec3(0,0,0),TA::Vec3(1000,1000,1000));
	TA::Physics::CreateInstance(world);
	TA::Physics& physics = TA::Physics::GetInstance();

	TA::StaticObject *pStaticObject=TA::StaticObject::CreateNew();
	TA::CollisionObjectAABBMesh *pStaticCollisionObject=TA::CollisionObjectAABBMesh::CreateNew();
	int vertex_counter=0;
	int triangle_counter=0;
	// create TA terrain from vertices and faces
	irr::scene::IMesh *mesh = planemesh->getMesh(0);
	const irr::u32 cnt = mesh->getMeshBufferCount();
	scene::IMeshBuffer* buf = mesh->getMeshBuffer(0);
	video::E_VERTEX_TYPE type = buf->getVertexType();
	irr::s32 vertexCount = buf->getVertexCount();
	irr::s32 indexCount = buf->getIndexCount();

	pStaticCollisionObject->Initialise(vertexCount,indexCount/3,indexCount);
	video::S3DVertex* vtx = (video::S3DVertex*)buf->getVertices();
	for(irr::s32 k=0;k<vertexCount;k++) { // copy to TA vertices buffer
		TA::Vec3 vec= CSceneNodeAnimatorTA::convertIrrlichtPos(vtx[k].Pos);
		pStaticCollisionObject->AddVertex(vec);
		vertex_counter++;
	}

	const irr::u16* indices = buf->getIndices();
	int polygonList[3];
	for (irr::s32 j=0; j<indexCount; j+=3) { // convert to TA triangles format
		polygonList[0]=indices[j+2];
		polygonList[1]=indices[j+1];
		polygonList[2]=indices[j+0];
		pStaticCollisionObject->AddPolygon(3,polygonList);
	}	

	pStaticCollisionObject->FinishedAddingGeometry();
	pStaticObject->Initialise(pStaticCollisionObject);
	pStaticCollisionObject->Release();
	physics.AddStaticObject(pStaticObject);
	pStaticObject->Release();

	for (int i=0; i<20;i++) { 
		for(int j=0;j<20-i;j++) {
			scene::IAnimatedMesh *mesh = smgr->getMesh("./cube.3ds");
			irr::f32 CUBESIZE = mesh->getBoundingBox().getExtent().X;
			core::vector3df pos(j*CUBESIZE+(i*(CUBESIZE/2)),i*CUBESIZE+(CUBESIZE/2.0f),0);
			scene::IAnimatedMeshSceneNode *cube = smgr->addAnimatedMeshSceneNode(mesh,0,-1,pos); 
			cube->addShadowVolumeSceneNode(-1,false);
			CSceneNodeAnimatorTA *tok = new CSceneNodeAnimatorTA(cube, 1.0f); 
			cube->addAnimator(tok); 
			tok->drop(); 
		}
	}

	scene::ICameraSceneNode* cam = smgr->addCameraSceneNodeFPS(0, 50.0f, 50.0f);
	cam->setPosition(core::vector3df(0,10,-10));
	cam->setTarget(core::vector3df(0,0,0));
	m_Device->getCursorControl()->setVisible(false);
	smgr->addLightSceneNode(0,core::vector3df(-50,50,-50),video::SColorf(1,1,1,1),1000);

	m_Device->setWindowCaption(L"Irrlicht + True Axis Physics Demo");
}

void CGame::Run(void) {

	video::IVideoDriver* driver = m_Device->getVideoDriver();
	scene::ISceneManager* smgr = m_Device->getSceneManager();
	TA::Physics& physics = TA::Physics::GetInstance();
	irr::ITimer *timer=m_Device->getTimer();

	irr::u32 prevtime = -1;
	while(m_Device->run() && m_Device->isWindowActive()) {
		irr::u32 now = timer->getTime();
		if(prevtime == -1) prevtime=now;
		irr::u32 timeMs = (now - prevtime);
		physics.Update(timeMs/1000.0f); // update TA	
		prevtime = now;

		driver->beginScene(true, true, video::SColor(255,0,0,0));
		smgr->drawAll();
		driver->endScene();
	}
}

void CGame::Fire(void) {
	scene::ISceneManager* smgr = m_Device->getSceneManager();
	scene::ICameraSceneNode* cam = smgr->getActiveCamera();
	scene::IAnimatedMeshSceneNode *cube = smgr->addAnimatedMeshSceneNode(smgr->getMesh("./cube.3ds"),0,-1,cam->getPosition()); 
	cube->addShadowVolumeSceneNode(-1,false);
	cube->setMaterialFlag(video::EMF_LIGHTING, false);
	cube->setMaterialTexture(0,m_Device->getVideoDriver()->getTexture("./rocket.png"));
	CSceneNodeAnimatorTA *tok = new CSceneNodeAnimatorTA(cube, 1000.0f, RIGID_BODY);
	cube->addAnimator(tok);
	tok->drop();	
	core::vector3df forward = (cam->getTarget()-cam->getPosition()).normalize();
	tok->m_pDynamicObject->SetLinearVelocity(CSceneNodeAnimatorTA::convertIrrlichtPos(forward*100.0f));
}


CGame::~CGame(void) {
	TA::Physics& physics = TA::Physics::GetInstance();
	physics.DestroyInstance();
	m_Device->drop();
}

bool CGame::OnEvent(const irr::SEvent &event) {
	if(event.EventType == EET_MOUSE_INPUT_EVENT) {
		if(event.MouseInput.Event == EMIE_LMOUSE_PRESSED_DOWN) {
			Fire();
			return true;
		}
	}

	if(event.EventType == EET_KEY_INPUT_EVENT) {
		if(event.KeyInput.Key == KEY_ESCAPE) {
			m_Device->closeDevice();
		}

	}
	return false;
}